#!/bin/bash
short_opts="h"
long_opts="out:,template:,serial:,disk:,size:,tar:,tar-d:,version:"
getopt_out=$(getopt --name "${0##*/}" \
    --options "${short_opts}" --long "${long_opts}" -- "$@") &&
    eval set -- "${getopt_out}" || { echo "BAD INVOCATION!"; usage; exit 1; }

usage() {
    cat <<EOM
${0##/} - Populated values in build temple.

    Required:
    --template      Template file
    --serial        The build serial
    --out           The output file
    --disk          The location of the disk for conversion
    --tar           Name of tar file
    --tar-d         Name of directory to tar up
    --size          Size of the disk
    --version       The version number of the distro
EOM
}

debug() { echo "${@}"; }
serial=$(date +%Y%m%d)
template_f="${0%/*}/img-azure.tmpl"
template_r="${0%/*}/img-azure-daily.tmpl"

while [ $# -ne 0 ]; do
  cur=${1}; next=${2};
  case "$cur" in
    --template)                 template_f=$2; shift;;
    --disk)                     disk_f=$2; shift;;
    --size)                     size=$2; shift;;
    --serial)                   serial=$2; shift;;
    --tar)                      tar_f=$2; shift;;
    --tar-d)                    tar_d=$2; shift;;
    --out)                      out_f=$2; shift;;
    --version)                  version=$2; shift;;
    --) shift; break;;
  esac
  shift;
done

working_image="${PWD}/${disk_f##*/}"
working_image="${working_image##*/}"
final_image_name="${working_image//.img/.vhd}"

fail() { echo "${@}" 2>&1; exit 1;}
fail_usage() { fail "Must define $@"; }

# Create the template file for image conversion
sed -e "s,%S,${serial},g" \
    -e "s,%v,${version},g" \
    ${template_f} > ${out_f} ||
        fail "Unable to write template file"

# Support per-suite addins
awk '/ADDIN_HERE/{n++}{print >"template" n ".txt" }' ${out_f}
addin_name="${template_f//.tmpl/}-${version}-addin.tmpl"
if [ -e "${addin_name}" ]; then
    cat template.txt ${addin_name} template[1-9].txt > ${out_f}
else
    cat template.txt template[1-9].txt > ${out_f}
fi

rm template*
sed -e "s,ADDIN_HERE,# END Addins,g" -i  ${out_f}

[ -e "${working_image}" ] && rm "${working_image}"
[ -e "${final_image_name}" ] && rm "${final_image_name}"
[ -e "${final_image_name}.pre" ] && rm "${final_image_name}.pre"

# Covert image to a RAW disk to work with
debug "Converting QCow2 to Raw Disk"
qemu-img convert -O raw "${disk_f}" "${working_image}" &&
    debug "Converted QCow2 to Raw disk for manipulation" ||
    fail  "Failed to convert QCow2 to Raw disk"

# Resize the raw image
debug "Truncating image to ${size}G"
truncate -s "${size}G" "${working_image}" &&
    debug "Truncated image at ${size}G" ||
    fail  "Failed to truncate disk image"

# Convert to VHD first, step 1 of cheap hack
# This is a cheap hack...half the time the next command
# will fail with "VERR_INVALID_PARAMETER", so this is the,
# er, workaround
debug "Converting to VHD"
vboxmanage convertfromraw --format VHD \
     "${working_image}" \
     "${final_image_name}.pre" &&
    debug "Converted raw disk to VHD" ||
    fail "Failed to convert raw image to VHD"

# Clone the disk to fixed, VHD for Azure
debug "Converting to VHD format from raw..."
debug ".....this might take a while...."
vboxmanage clonehd --format VHD --variant Fixed \
    "${final_image_name}.pre" \
    "${final_image_name}" &&
    debug "Converted raw disk to VHD format using VirtualBox" ||
    fail  "Failed to convert raw image to VHD disk!"
set +x

rm "${working_image}" "${final_image_name}.pre"

debug "Image Characteristics:"
vboxmanage showhdinfo "${final_image_name}"

debug "Image converted; configuration ready"

if [ -n "${tar_d}" ]; then
   tar -C "${tar_d}" -cf "${tar_f}" . &&
        debug "TAR'd up ${tar_d}" ||
        fail  "Failed to tar up ${tar_d}"
fi
exit 0
