#!/bin/bash
fail() { [ $# -eq 0 ] || echo "$@"; exit 1; }

umask 022
# Pre-setup: Read the build properties from the previous build
# and discard what we don't want
set -x
source build_properties

mv build_properties parent_build_properties
mv build.log parent_build.log
rm "${SUITE}-build.sh"
set +x

echo "-------------------"
echo " Distro: ${SUITE}"
echo " Serial: ${SERIAL}"
echo " Type:   ${BUILD_TYPE}"
echo "-------------------"

# Variables
disk_name="${SUITE}-server-cloudimg-amd64-disk1.img"
vhd_name="${PWD}/${SUITE}-server-cloudimg-amd64-disk1.vhd"
disk_root="/srv/ec2-images/${SUITE}/${SERIAL}"
raw_disk="${PWD}/results.raw"
launch_config="${PWD}/launch_config.sh"
register_config="${PWD}/register_config.sh"
pkg_tar="${PWD}/pkg.tar"
pkg_tar_d="${kvm_builder}/azure_pkgs"


# Setup the configuration
${kvm_builder}/azure_config.sh \
    --version $(${kvm_builder}/ubuntu-adj2version ${SUITE}) \
    --disk "${disk_root}/${disk_name}" \
    --serial "${SERIAL}" \
    --size 30 \
    --out "${launch_config}" ||
        fail "Failed to configure instance runtime"

# Launch KVM to do the work
${kvm_builder}/launch_kvm.sh \
    --id ${BUILD_ID} \
    --user-data "${launch_config}" \
    --cloud-config "${kvm_builder}/cloud-azure.cfg" \
    --extra-disk "${vhd_name}" \
    --raw-disk "${WORKSPACE}/${SUITE}-output.raw" \
    --raw-size 2 \
    --img-url /srv/builder/images/precise-vda-root-20120726.img ||
        fail "KVM instance failed to build image."

# Clean up
rm -rf "${kvm_builder}"
rm "${WORKSPACE}/${SUITE}-output.raw"
rm -rf "${kvm_builder}"

# Upload the latest image
azure_pre="/srv/builder/azure/cronrun azure"
stamp=$(date +%s)
upload_name="${SUITE}-daily-${SERIAL}-${stamp}"
ln -s "${vhd_name}" "${WORKSPACE}/${upload_name}"
${azure_pre} vm image create -v -o linux -f \
    -u "http://ubuntunortheurope.blob.core.windows.net/daily-vhd/${upload_name}" \
    --description "${SUITE} daily build ${BUILD_ID} ${stamp}" \
    "${upload_name}" "${WORKSPACE}/${upload_name}" ||
        echo "Failed to register image as ${upload_name}"

# Log it
cat << EOF > "${WORKSPACE}/register.txt"
SUITE=${SUITE}
SERIAL=${SERIAL}
REGISTRATION=${upload_name}
URL=http://ubuntunortheurope.blob.core.windows.net/daily-vhd/${upload_name}
BUILD_ID=${BUILD_ID}

EOF

rm "${WORKSPACE}/${upload_name}"

# Get the current images that match the current distro
images=($($azure_pre vm image list | grep ${SUITE} | awk '/data:/ {print$2}' | sort -r))

# Keep only the last five images
echo "Purging extra images"
image_count="${#images[@]}"
[ "${image_count}" -gt 5 ] && {
    for item in $(seq 5 ${image_count})
    do
        echo "De-registering ${images[$item]}"
        ${azure_pre} vm image delete -v ${images[$item]} ||
            echo "failed to de-register image ${images[$item]}"
    done
} || echo "No images to de-register at this time"

# Archive the bzip2 file
echo "Archiving final image"
bzip2 "${vhd_name}" ||
    fail "Failed to compress image"

[ -e "${disk_root}/azure" ] || mkdir -p "${disk_root}/azure"
echo "Moving Image into place"
mv "${vhd_name}.bz2" "${disk_root}/azure" ||
    fail "Failed to move final image to final place"

chown jenkins:serverteam-jenkins "${disk_root}/azure" ||
    fail "Failed to assign ownership final image"

chmod 0644 "${disk_root}/azure/${SUITE}-server-cloudimg-amd64-disk1.vhd.bz2" ||
    fail "Failed to assign permissions to final image"
