#!/bin/bash

# Read in the common files
source "${kvm}/functions/common"
source "${kvm}/functions/retry"
export HOME=${WORKSPACE}

# Apply the build stuff
[ -e "build_properties" ] &&
    source build_properties ||
    fail "Failed to read build_properties. I don't know what I'm doing!"

rm {failed,success} || /bin/true

# Copy the target disk image
ARCH_TYPE=${ARCH_TYPE:-$ARCH}
disk_orig="${SUITE}-server-cloudimg-${ARCH_TYPE}-disk1.img"
disk_cp="${disk_orig//$ARCH_TYPE/$ARCH_TYPE-vagrant}"
disk_root="/srv/ec2-images/${SUITE}/${SERIAL:-current}"
disk_working="${WORKSPACE}/${disk_cp}"
final_disk="${WORKSPACE}/box-disk1.vdi"
final_location="/srv/ec2-images/vagrant/${SUITE}/${SERIAL}"
box_name="${disk_working//.img/.box}"
raw_f="${WORKSPACE}/raw_f-$(date +%s).img"

[ -e "${final_location}/${box_name}" -a "${REBUILD}" != "true" ] && exit 0

cp "${disk_root}/${disk_orig}" "${disk_working}" ||
    fail "Unable to copy ${disk_orig} from ${disk_root}"

# Launch KVM to do the work
${kvm}/launch_kvm.sh \
        --id "${ARCH_TYPE}-${BUILD_ID}" \
        --user-data "${kvm}/cloud-vps.cfg" \
        --cloud-config "${kvm}/img-vagrant.tmpl" \
        --extra-disk "${disk_working}" \
        --disk-gb 1 \
        --raw-disk "${raw_f}" \
        --raw-size 1 \
        --img-url /srv/builder/images/precise-vda-root-20120726.img ||
            fail "KVM instance failed to build image."

# Covert to VMDK
qemu-img convert -O raw ${disk_working} ${disk_working//.img/.raw}
truncate -s 40G ${disk_working//.img/.raw}

vboxmanage convertfromraw \
    --format vdi \
    ${disk_working//.img/.raw} ${final_disk}

# Create the VM
vmname="ubuntu-cloudimg-${SUITE}-vagrant-${ARCH_TYPE}"
vboxmanage modifyhd --compact ${final_disk}

vboxmanage createvm \
    --name ${vmname} \
    --ostype Ubuntu_64 \
    --register

vboxmanage modifyvm ${vmname} \
    --memory 512 \
    --boot1 disk \
    --boot2 none \
    --boot3 none \
    --boot4 none \
    --vram 12 \
    --pae off \
    --rtcuseutc on
#    --natnet1 default \

vboxmanage modifyvm ${vmname} --natpf1 "guestssh,tcp,,2222,,22"

vboxmanage storagectl "${vmname}" \
    --name "SATA Controller" \
    --add sata \
    --controller IntelAhci \
    --sataportcount 1 \
    --hostiocache off

vboxmanage storageattach ${vmname} \
    --storagectl "SATA Controller" \
    --port 0 \
    --device 0 \
    --type hdd \
    --medium ${final_disk}

vboxmanage setextradata ${vmname} installdate ${serial}
vboxmanage setextradata ${vmname} flavor "Ubuntu Server Cloud Image Vagrant"
vboxmanage setextradata ${vmname} supported false

mkdir box

let wait_time=$RANDOM%90
sleep $wait_time # Make build collisions a bit harder

retry 3 60 vboxmanage export ${vmname} --output box/box.ovf

# Create the Vagrant file
#macaddr="02:$(openssl rand -hex 5)"
macaddr=$(awk '-F"' '/<Adapter slot="0" enabled="true"/ {print$6}' ${WORKSPACE}/box/box.ovf)
cat << EOF > ${WORKSPACE}/box/Vagrantfile
Vagrant::Config.run do |config|
  # This Vagrantfile is auto-generated by 'vagrant package' to contain
  # the MAC address of the box. Custom configuration should be placed in
  # the actual 'Vagrantfile' in this box.
  config.vm.base_mac = "${macaddr}"
end

# Load include vagrant file if it exists after the auto-generated
# so it can override any of the settings
include_vagrantfile = File.expand_path("../include/_Vagrantfile", __FILE__)
load include_vagrantfile if File.exist?(include_vagrantfile)
EOF

# Now pack it all up....
tar -C ${WORKSPACE}/box -Scvf ${box_name} box.ovf Vagrantfile box-disk1.vmdk ||
        fail "Unable to create box file"

# Some minor cleanup
rm ${disk_working} ${disk_working//.img/.raw} || /bin/true
rm -rf ${WORKSPACE}/box *.vdi
rm "${raw_f}" || /bin/true

# Put the box in place
mkdir -p "${final_location}" ||
    fail "Unable to create the vagrant image location"

cp ${box_name} ${final_location} ||
    fail "Failed to place vagrant image in final home"

# Update the current link
current_l="/srv/ec2-images/vagrant/${SUITE}/current"
[ -e "${current_l}" ] && rm "${current_l}"
( cd "/srv/ec2-images/vagrant/${SUITE}" && ln -s "${SERIAL}" current )

# Cleanup old builds
builds=($(find /srv/ec2-images/vagrant/${SUITE} -mindepth 1 -maxdepth 1 -type d | sort -r))
build_count="${#builds[@]}"

echo "------------------------"
echo "Clean-up for prior builds"
echo "Found ${build_count} builds for consideration"

for b in ${builds[@]}
do
    echo " - found build ${b}"
done
echo ""

[ "${build_count}" -gt 4 ] && {
    for item in $(seq 4 ${build_count})
    do
        [ -e "${builds[$item]}" ] && {
            echo "Removing build ${builds[$item]} for deletion"
            rm -rf ${builds[$item]} ||
                echo "Failed to remove build ${builds[$item]}"
        }
    done

    for item in $(seq 0 3)
    do
        [ -e "${builds[$item]}" ] &&
            echo "Preserving build ${builds[$item]}"
    done

} || echo "No builds marked for removal"

