#!/bin/bash

fail() { [ $# -eq 0 ] || echo "$@"; exit 1; }

find . -iname "*build_properties" | xargs -I FILE cp FILE .
[ -e "build_properties" ] &&
    source build_properties ||
    fail "Failed to read build_properties. I don't know what I'm doing!"

set -x
out_f="${WORKSPACE}/maas-${SUITE}-${STREAM}-config.sh"
raw_f="${WORKSPACE}/${SUITE}-output.raw"
query_t="${WORKSPACE}/cloud-images-query.tar"
base_name="${SUITE}-server-cloudimg"
rel_base_name="ubuntu-$(ubuntu-adj2version ${SUITE})-${stream//-/}-server-cloudimg"

export maas_branch="${MAAS_BRANCH:-http://bazaar.launchpad.net/~smoser/maas/maas.ubuntu.com.images-ephemeral}"

case "${STREAM}" in
    release)  build_f="/srv/ec2-images/releases/${SUITE}/release-${SERIAL}";
              base_name=${rel_base_name};
              out_d="/srv/maas-images/ephemeral/releases/${SUITE}/release-${SERIAL}"
              ;;
    daily)    build_f="/srv/ec2-images/${SUITE}/${SERIAL}";
              out_d="/srv/maas-images/ephemeral/daily/${SUITE}/${SERIAL}";
              ;;
    alpha*|beta*)  build_f="/srv/ec2-images/releases/${SUITE}/${STREAM}";
              base_name=${rel_base_name};
              out_d="/srv/maas-images/ephemeral/release/${SUITE}/${STREAM}";
              ;;
    *)        fail "Unknown stream ${STREAM}.";;
esac

final_out_d="${out_d}"

[ -e "${final_out_d}" -a "${REBUILD:-false}" = "false" ] &&
    fail "Build already exists. Rebuild is set to false. Failing this build"

# Tar up query for use in-image
tar cvf ${query_t} \
    /srv/ec2-images/query \
    ${build_f}  \
    --exclude "*img" --exclude "*azure*" --exclude "*html" \
    --exclude "*armel*" --exclude "*root.tar.gz" \
    --exclude "*floppy" ||
    fail "Failed to pack up build elements for MAAS builder"

# Generate the template file
maas_config.sh \
    --distro "${SUITE}" \
    --stream "${STREAM}" \
    --template "${kvm_builder}/img-maas.tmpl" \
    --base-name "${base_name}" \
    --local "${build_f}" \
    --serial "${SERIAL}" \
    --out "${out_f}" \
    --out_d "${out_d}" ||
    fail "Failed to configure KVM instance for building"
set +x

# Launch KVM to do the work
launch_kvm.sh \
    --id ${BUILD_ID} \
    --user-data "${out_f}" \
    --cloud-config "${kvm_builder}/cloud-maas.cfg" \
    --extra-disk "${query_t}" \
    --disk-gb 30 \
    --raw-disk "${raw_f}" \
    --raw-size 10 \
    --img-url /srv/builder/images/precise-vda-root-20120726.img ||
        fail "KVM instance failed to build image."

# Extract the result set
tar -xvvf "${raw_f}" ||
    fail "Failed to extract information from instance"

# Extracted reslts should be here
[ ! -e "${WORKSPACE}/${out_d}" ] && fail "Expected result directory is missing"

# Checksum the results (and sign 'em)
export CDIMAGE_ROOT="/srv/builder/vmbuilder/cdimage"
/srv/builder/vmbuilder/bin/cronrun checksum-directory "${WORKSPACE}/${out_d}" ||
    fail "Failed to create checksums and GPG signatures"

set -x
# Put the bits where they go...
mkdir -p "${final_out_d}" &&
    cp -a ${WORKSPACE}${out_d}/* "${final_out_d}" &&
    echo "Copied bits to final location ${final_out_d}" ||
    fail "Unable to copy build bits to final location"

# Produce build-info
cat << EOF > "${final_out_d}/build-info.txt"
serial=${SERIAL}
orig_prefix=${SUITE}-ephemeral-maas
suite=${SUITE}
build_name=ephemeral
EOF

# Generate the Query2 tree
src_tree="${WORKSPACE}/maas_src"
bzr branch "${maas_branch}" "${src_tree}"
${src_tree}/tree2query \
    --commit-msg "Build ${BUILD_ID}" \
    --namespace maas \
    /srv/maas-images

# Update current
if [ "${STREAM}" = "daily" ]; then
    cur_d="/srv/maas-images/ephemeral/daily/${SUITE}/current"
    [ -e "${cur_d}" ] && rm "${cur_d}"
    ln -s "${final_out_d}" "${cur_d}" ||
        echo "Failed to update ${cur_d}"
fi


# Remove the results
rm "${raw_f}" ||
    fail "Failed to clean up files!"

