#!/bin/bash

error() { echo "$@" 1>&2; }
fail() { error "$@"; sleep 120; exit 1; }
debug() { error "$(date -R):" "$@"; }

#Look for registration text. This contains the information
#about what to test. This is important.
registration="${WORKSPACE}/register.txt"
[ -e "${registration}" ] &&
    source ${registration} ||
        fail "Registration information is missing. Failing."

#Setup control variables
test_suite=${1}
vm_name_base=${SUITE}-test-$(date +%Y%m%s)-${BUILD_NUMBER}
def_user="ubutest"
def_pass="$(openssl rand -base64 16 | awk '-F=' '{print$1}')"
running=()  # collection of images running

#Setup paths for Azure command
NODE_LIB="${WORKSPACE}/usr/local/lib/node_modules/azure"
LOCAL_LIB="${WORKSPACE}/usr/local/lib"
LOCAL_LOCAL="${WORKSPACE}/usr/local"
LOCAL_BIN="${WORKSPACE}/usr/local/bin"
azure_cmd="${WORKSPACE}/usr/local/bin/azure"

#Reset some pathing
export HOME="${WORKSPACE}"
export PATH="${NODE_LIB}:${LOCAL_LIB}:${LOCAL_LOCAL}:${LOCAL_BIN}:${PATH}"

#Sanity checking!
[ -z "${SUITE}" ] && fail "Suite is undefined!"
[ -z "${SERIAL}" ] && fail "Serial is undefined!"
[ -z "${REGISTRATION}" ] && fail "Registration name is undefined"
[ -z "${URL}" ] && fail "URL is undefined"

#Output something pretty
set +x
cat <<EOM
_________________________________________________________________

*****************************************************************

TESTING:
    Suite: ${SUITE}
    Serial: ${SERIAL}
    Registration: ${REGISTRATION}
    Disk URL: ${URL}

Test: ${test_suite}

*****************************************************************
_________________________________________________________________
EOM
set -x

#Retry command
retry() {
    local trycount=${1} sleep=${2}
    shift; shift;
    local i=0 smsg=" sleeping ${sleep}: $*" ret=0
    for((i=0;i<${trycount};i++)); do
        "$@" && return 0
        ret=$?
        [ $(($i+1)) -eq ${trycount} ] && smsg=""
        debug 1 "Warning: cmd failed [try $(($i+1))/${trycount}].${smsg}"
        sleep $sleep
    done
    return $ret
}

#Kill instance
terminate() {
    local disk=$($azure_cmd vm disk list  2> /dev/null | grep ${1} | awk '{print$2}')

    retry 3 30 ${azure_cmd} vm delete ${1} ||
        error "Failed to delete VM ${1}"

    error "Sleeping for two minutes to allow disk to settle"
    sleep 120

    [ -n "${disk}" ] &&
        retry 3 30 ${azure_cmd} vm disk delete ${disk} ||
            error "Failed to delete disk ${disk}"

    #remove it from list of running images
    running=(${running[@]//${1}/})
}

#Kill all instances
terminate_all() {
    for img in ${running[@]}
    do
        terminate ${img}
    done
}

#Make sure any exit, kills all running instances
trap terminate_all EXIT

#Launch an instance
run_instance() {
    local vmname=${1}
    local extras=(${@})
    unset extras[0];

    local run_cmd=(
           ${azure_cmd}
           vm
           create
           ${vmname}
           ${REGISTRATION}
           --location 'North Europe'
           -v
           -e 22
           ${def_user}
           ${def_pass}
        )

    [ -n "${extras[@]}" ] &&
        run_cmd+=(${extras[@]})

    "${run_cmd[@]}" || fail "Unable to launch instance"
    running+=(${1})
}

check_instance() {
    local status=$($azure_cmd vm show ${1} | awk '-F"' '/InstanceStatus/ {print$2}')
    [ "${status}" = "ReadyRole" ] && return 0
    [ "${status}" = "FailedProvisioning" ] && fail "Failed provisioning"
    return 1
}

get_dns() {
    dns=$($azure_cmd vm show ${1} |
                 awk '-F"' '/DNSName/  {print$2}')

    [ -n "${dns}" ] && echo "${dns}" ||
        fail "Unable to determine DNS name for ${1}"
}

wait_for() {
    local sleeptime=${1}
    local max_count=${2}
    local command=(${@})
    unset command[0]; unset command[1];
    local count=0

    while $(sleep ${sleeptime}); do
        ${command[@]} && return 0
        count=$(expr ${count} + 1 )

        if [ "${count}" -ge "${max_count}" ]; then
            return 1
        fi
    done
}



# Test password-based SSH
password_ssh() {
    which sshpass 2> /dev/null &>1 ||
        sudo apt-get -y install sshpass ||
            fail "Required program, sshpass, is not installed"

    # create instance
    local vmname="${vm_name_base}-pass"
    run_instance ${vmname}

    wait_for 15 60 check_instance ${vmname} ||
        fail "Instance failed to initialize in time"

    local dnsname=$(get_dns ${vmname})

    local ssh_pass="ssh pass -p ${def_pass} ssh ${def_user}@${dnsname}"

    # Check if things are working
    local ssh_check=()
    ssh_check+=(${ssh_pass} /bin/true)
    retry 5 60 ${ssh_cmd[@]} &&
        error "Instance passed boot test" ||
        fail "Test instance creation failed for ${vmname}"
}

# Test passwordless ssh
key_ssh() {

    # This doesn't work at this time
    exit 0

    # SSH key creation for Azure posted here:
    # https://www.windowsazure.com/en-us/manage/linux/how-to-guides/ssh-into-linux/
    openssl req \
        -x509 \
        -batch \
        -nodes \
        -days 365 \
        -newkey rsa:2048 \
        -keyout myPrivateKey.key \
        -out myCert.pem
    chmod 600 myPrivateKey.key
    openssl  x509 -outform der -in myCert.pem -out cert.pem ||
        fail "Failed to generate SSH certificate"

    ssh-keygen -e -f myPrivateKey.key > openssh.pem ||
        fail "Unable to create public SSH certificate"

    # Launch the instance
    vmname="${vm_name_base}-kssh"
    vmargs="-t ${WORKSPACE}/cert.pem"
    run_instance ${vmname} ${vmargs}

    wait_for 15 60 check_instance ${vmname} ||
        fail "Instance failed to initialize in time"

    dnsname=$(get_dns ${vmname})

    # Check if things are working
    ssh_check="ssh -i ${WORKSPACE}/myPrivateKey.key ${def_user}@${dnsname} /bin/true"
    retry 5 60 ${ssh_check} ||
        fail "Failed SSH key check"

}

# Look at image characteristics
chacteristics() {
    exit 0
}

case ${1} in
    Password-SSH)       password_ssh;;
    Key-SSH)            key_ssh;;
    Characteristics)    chacteristics;;
esac

